# -*- coding: utf-8 -*-
from plone.app.linkintegrity.utils import ensure_intid
from plone.app.linkintegrity.utils import referencedRelationship
from plone.app.uuid.utils import uuidToObject
try:
    from Products.Archetypes.config import REFERENCE_CATALOG
except ImportError:
    REFERENCE_CATALOG = "reference_catalog"
from Products.CMFCore.utils import getToolByName
from z3c.relationfield import RelationValue
from z3c.relationfield.event import _setRelation
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import logging
log = logging.getLogger(__name__)


def migrate_linkintegrity_relations(context):
    """Migrate linkintegrity-relation from reference_catalog to zc.relation.
    """
    reference_catalog = getToolByName(context, REFERENCE_CATALOG, None)
    intids = getUtility(IIntIds)
    if reference_catalog is not None:
        # Only handle linkintegrity-relations ('isReferencing').
        # [:] copies the full result list to make sure
        # it won't change while we delete references below
        for brain in reference_catalog(relationship=referencedRelationship)[:]:
            try:
                source_obj = uuidToObject(brain.sourceUID)
                target_obj = uuidToObject(brain.targetUID)
            except AttributeError:
                source_obj = target_obj = None
            if source_obj is None or target_obj is None:
                # reference_catalog may be inconsistent
                log.info('Cannot delete relation since the relation_catalog is inconsistent.')   # noqa: E501
                continue
            # Delete old reference
            reference_catalog.deleteReference(
                source_obj, target_obj, relationship=referencedRelationship)

            # Trigger the recreation of linkintegrity-relation in
            # the relation_catalog (zc.relation)
            target_id = ensure_intid(target_obj, intids)
            if target_id is None:
                continue
            rel = RelationValue(target_id)
            _setRelation(source_obj, referencedRelationship, rel)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from datetime import datetime
from datetime import timedelta
from plone.app.linkintegrity.handlers import modifiedContent
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from transaction import savepoint
from zExceptions import NotFound

import logging
logger = logging.getLogger(__name__)


class UpdateView(BrowserView):
    """Iterate over all catalogued items and update linkintegrity-information.
    """

    def __call__(self):
        context = aq_inner(self.context)
        request = aq_inner(self.request)
        if 'update' in request.form or 'delete_all' in request.form:
            starttime = datetime.now()
            count = self.update()
            duration = timedelta(seconds=(datetime.now() - starttime).seconds)
            msg = _(
                u'linkintegrity_update_info',
                default=u'Link integrity information updated for ${count} ' +
                        u'items in ${time} seconds.',
                mapping={'count': count, 'time': str(duration)},
            )
            IStatusMessage(request).add(msg, type='info')
            msg = 'Updated {0} items in {1} seconds'.format(
                count,
                str(duration),
            )
            logger.info(msg)
            request.RESPONSE.redirect(getToolByName(context, 'portal_url')())
        elif 'cancel' in request.form:
            msg = _(u'Update cancelled.')
            IStatusMessage(request).add(msg, type='info')
            request.RESPONSE.redirect(getToolByName(context, 'portal_url')())
        else:
            return self.index()

    def update(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        count = 0

        for brain in catalog():
            try:
                obj = brain.getObject()
            except (AttributeError, NotFound, KeyError):
                msg = 'Catalog inconsistency: {0} not found!'
                logger.error(msg.format(brain.getPath()), exc_info=1)
                continue
            try:
                modifiedContent(obj, 'dummy event parameter')
                count += 1
            except Exception:
                msg = 'Error updating linkintegrity-info for {0}.'
                logger.error(msg.format(obj.absolute_url()), exc_info=1)
            if count % 1000 == 0:
                savepoint(optimistic=True)
        return count



# -*- coding: utf-8 -*-
from plone.app.linkintegrity.tests.base import ATBaseTestCase
from plone.app.linkintegrity.upgrades import migrate_linkintegrity_relations
from plone.app.linkintegrity.utils import hasIncomingLinks
from plone.app.linkintegrity.utils import referencedRelationship
try:
    from Products.Archetypes.interfaces import IReferenceable
    HAS_AT = True
except ImportError:
    HAS_AT = False
import unittest


class ReferenceMigrationATTestCase(ATBaseTestCase):
    """Reference migration testcase for at content types"""

    @unittest.skipUnless(
        HAS_AT, 'Archetypes are not installed. Skipping migration tests')
    def test_upgrade(self):
        doc3 = self.portal['doc3']
        doc1 = self.portal['doc1']
        self.assertTrue(IReferenceable.providedBy(doc3))
        doc3.setText('<a href="doc1">doc1</a>', mimetype='text/html')
        doc3.addReference(doc1, relationship=referencedRelationship)
        self.assertFalse(hasIncomingLinks(doc1))
        self.assertFalse(hasIncomingLinks(doc3))
        migrate_linkintegrity_relations(self.portal)
        self.assertTrue(hasIncomingLinks(doc1))
        self.assertFalse(hasIncomingLinks(doc3))



# -*- coding: utf-8 -*-
from plone.app.linkintegrity import testing
from plone.app.relationfield.behavior import IRelatedItems
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.textfield import RichTextValue
from plone.testing.z2 import Browser
from z3c.form.interfaces import IFormLayer
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.lifecycleevent import modified

import unittest


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        alsoProvides(self.request, IFormLayer)

        # Get a testbrowser
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.browser.addHeader('Referer', self.portal.absolute_url())
        self.browser.addHeader(
            'Authorization',
            'Basic {0:s}:{1:s}'.format(TEST_USER_NAME, TEST_USER_PASSWORD))

        # Do an initial page load to make sure the bundles get compiled
        # (which currently commits a transaction)
        # before we render exception views
        self.browser.open(self.portal.absolute_url())

        setRoles(self.portal, TEST_USER_ID, ['Manager', ])

    def _get_token(self, obj):
        return getMultiAdapter(
            (obj, self.request), name='authenticator').token()


class DXBaseTestCase(BaseTestCase):
    """Base testcase for testing Dexterity content types"""

    layer = testing.PLONE_APP_LINKINTEGRITY_DX_FUNCTIONAL_TESTING

    def _set_text(self, obj, text):
        obj.text = RichTextValue(text)
        modified(obj)

    def _get_text(self, obj):
        return obj.text.raw

    def _set_related_items(self, obj, items):
        assert IRelatedItems.providedBy(obj)
        setattr(obj, 'relatedItems', items)
        modified(obj)

    def _get_related_items(self, obj):
        return obj.relatedItems


class ATBaseTestCase(BaseTestCase):
    """Base testcase for testing Archetypes content types"""

    layer = testing.PLONE_APP_LINKINTEGRITY_AT_FUNCTIONAL_TESTING

    def _set_text(self, obj, text):
        obj.setText(text, mimetype='text/html')
        modified(obj)

    def _get_text(self, obj):
        # This is the equivalent to obj.text in dexterity. No transforms,
        # no rewritten relative urls
        return obj.getText(raw=1).raw

    def _set_related_items(self, obj, items):
        obj.setRelatedItems(items)
        modified(obj)

    def _get_related_items(self, obj):
        return obj.getRelatedItems()



# -*- coding: utf-8 -*-
from plone.app.linkintegrity.handlers import findObject
from plone.app.linkintegrity.tests.base import ATBaseTestCase
from plone.app.linkintegrity.tests.base import DXBaseTestCase


class FindObjectTests:
    """ testing the handlers.findObject function """

    def test_relative_to_portal_root_1(self):
        obj, components = findObject(self.portal.doc1, '/plone/doc2')
        self.assertEqual(obj.absolute_url_path(), '/plone/doc2')
        self.assertEqual(components, '')

    def test_relative_to_portal_root_2(self):
        # Prevent regression.
        # See https://github.com/plone/plone.app.linkintegrity/pull/17
        obj, components = findObject(self.portal.doc1, '/doc2')
        self.assertEqual(obj.absolute_url_path(), '/plone/doc2')
        self.assertEqual(components, '')

    def test_webserver_rewrites_portal_name(self):
        # test the case where a webserver rewrites the portal name,
        # e.g. for Apache:
        # RewriteRule ^/wssitename(.*)$ http://localhost:8080/VirtualHostBase/http/my.domain.com:80/plonesitename/VirtualHostRoot/_vh_wssitename$1  # noqa
        self.portal.REQUEST.other['VirtualRootPhysicalPath'] = ('', 'plone')
        self.portal.REQUEST._script = ['plone_foo']
        obj, components = findObject(self.portal.doc1, '/plone_foo/doc2')
        self.assertEqual(obj.absolute_url_path(), '/plone_foo/doc2')
        self.assertEqual(obj.getPhysicalPath(), ('', 'plone', 'doc2'))
        self.assertEqual(components, '')


class ReferenceGenerationDXTestCase(DXBaseTestCase, FindObjectTests):
    """findObject testcase for dx content types"""


class ReferenceGenerationATTestCase(ATBaseTestCase, FindObjectTests):
    """findObject testcase for at content types"""
